/** Double-include prevention */
#if defined _togscrim_included_
  #endinput
#endif
#define _togscrim_included_

/**
 * Used to find out which team (CT or Terr) a given team group (1 or 2) currently is.
 *
 * @param	iTeamGroup			Team group (1 or 2) to get the current team for.
 * 
 * @return Current team number (2=Terr, 3=CT) for a given team group, else 0 if no match in progress or anything other than 1 or 2 is passed to native.
 */
native int TScrim_GetTeamGrpCurrTeam(int iTeamGroup);
 
/**
 * Used to get current match ID.
 * 
 * @return Current match ID, or 0 if no match in progress.
 */
native int TScrim_GetMatchID();
 
/**
 * Used to get current match status.
 * 
 * @return Current match status, or 0 if no match in progress.
 * 
 * STATUS_WARMUP 				0
 * STATUS_HALFTIME 				1
 * STATUS_CAPTAINSROUND 		2
 * STATUS_CAPTAINSROUND2 		3
 * STATUS_PICKINGTEAMS 			4
 * STATUS_TEAMSSET				5	//only used briefly to signify it is ok to show the captains menu using the cmd
 * STATUS_POSTCAPTAINSROUND 	6	//captains rnd 1 winner has selected map or sides - map menu now being shown
 * STATUS_MATCHSTART 			7	//map selected - changed to new map, if applicable
 * STATUS_SIDESPICKED 			8	//same as STATUS_MATCHSTART, but after sides picked
 * STATUS_MATCHLIVE 			9
 * STATUS_OVERTIME_START		10
 * STATUS_OVERTIME_LIVE 		11
 * STATUS_OVERTIME_HALF 		12
 * 
 */
native int TScrim_GetMatchStatus();

/* 
 * Called when a new match is started (may be starting captains round, else skipping captains round may be forced).
 * 
 * @return	None.
 */ 
forward void TScrim_MatchStart();

/* 
 * Called when a match ends.
 * 
 * @return	None.
 *
 * MATCHEND_NOTSET 				0
 * MATCHEND_TEAM1WINS 			1
 * MATCHEND_TEAM2WINS 			2
 * MATCHEND_TIEGAME 			3
 * MATCHEND_ADMINCANCEL 		4
 * MATCHEND_VOTECANCEL 			5
 * MATCHEND_TEAM1SURRENDER		6
 * MATCHEND_TEAM2SURRENDER		7
 * MATCHEND_CAPTAINDISCONNECTED 8
 *
 */ 
forward void TScrim_MatchEnd(int iMatchEndReason);

/* 
 * Called when a match goes live (start of match, second half, and overtimes).
 * 
 * @return	None.
 */ 
forward void TScrim_MatchLive();

/* 
 * Called at the start of pause for halftime.
 * 
 * @return	None.
 */ 
forward void TScrim_Halftime();

/* 
 * Called at the start of pause for overtime.
 * 
 * @return	None.
 */ 
forward void TScrim_Overtime();

/* 
 * Called at the start of pause for overtime half.
 * 
 * @return	None.
 */ 
forward void TScrim_OTHalf();

/* 
 * Called at the start of the 1st captains round. Note: Captains round can be skipped by admin commands, so this may not be called.
 * 
 * @return	None.
 */ 
forward void TScrim_CaptainsRnd1();

/* 
 * Called at the start of the 2nd captains round. Note: Captains round can be skipped by admin commands, so this may not be called.
 * 
 * @return	None.
 */ 
forward void TScrim_CaptainsRnd2();

/* 
 * Called after captains round finishes. Note: Captains round can be skipped by admin commands, so this may not be called.
 * 
 * @return	None.
 */ 
forward void TScrim_PostCaptRnd();


public SharedPlugin __pl_togscrim =
{
	name = "togscrim",
	file = "togscrim.smx",
#if defined REQUIRE_PLUGIN
	required = 1
#else
	required = 0
#endif
};

#if !defined REQUIRE_PLUGIN
public void __pl_togscrim_SetNTVOptional()
{
	MarkNativeAsOptional("TScrim_GetTeamGrpCurrTeam");
	MarkNativeAsOptional("TScrim_GetMatchID");
	MarkNativeAsOptional("TScrim_GetMatchStatus");
}
#endif